#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 Connect to and Export from DB

 @author East Japan Accounting Center Co., Ltd

 (c)2021 The University of Aizu
 This software is released under the New BSD License.
"""
import logging

import pymongo
from enum import IntFlag, auto

logger = logging.getLogger(__name__)

# Query key
KEY_DATETIME_STR = "datetime.datetime"
KEY_NATURAL_STR = "$natural"
# Index
INDEX_DATETIME = "datetime.datetime_1"


class MongoQueryOrder(IntFlag):
    REGISTER_ASC = auto()
    REGISTER_DESC = auto()
    CHRONOLOGICAL_ASC = auto()
    CHRONOLOGICAL_DESC = auto()


class MongoDB:
    """
    MongoDB operation(Connection, File export, ...)
    """

    def __init__(self, host, port, db_name, collection_name=""):
        self._client = None
        self._db = None
        self._host = host
        self._port = port
        self._db_name = db_name
        self._collection_name = collection_name

    def connection(self, host="localhost", port=27017):
        """
        Connect to specified DB

        Parameters
        ----------
        host: string(Optional, Default: localhost)
            Host name on which to connect
        port: int(Optional, Default: 27017)
            Port number on which to connect

        Returns
        -------
        DB instance
        """
        try:
            self._client = pymongo.MongoClient(host=host, port=port)
        except Exception:
            logger.error("Unable to connect to NoSQL.")
        return self._client

    # def _start_session(self, client):

    #     self._session = client.start_session()
    #     self._session.start_transaction()

    def get_document(self, db_name, collection_name, query):
        """
        Data corresponding to the query

        Parameters
        ----------
        db_name: string
            DB name
        collection_name: string
            Collection name
        query: dictionary
            Query

        Returns
        -------
        df: pandas.core.frame.DataFrame
            Dataframe
        doc: document list
        """

        self._db_name = db_name
        self._collection_name = collection_name
        doc = None
        # Do NOT export if the DB, collection are not specified
        if len(db_name) == 0 or len(collection_name) == 0:
            logger.error("DB param is empty.")
            return doc
        # Querying for more than one document
        try:
            if db_name in self._client.list_database_names() and collection_name in self._client[db_name].list_collection_names():
                self._db = self._client[db_name]
                co = self._db[collection_name]
                doc = list(co.find(query).sort(KEY_NATURAL_STR, 1))
            else:
                logger.error("Unable to search from NoSQL")
        except Exception:
            logger.error("Unable to search from NoSQL")
        return doc

    def insert_document(self, db_name, collection_name, doc):
        """
        Insert the one document in the specified DB

        Parameters
        ----------
        db_name: string
            DB name
        collection_name: string
            Collection name
        doc: dictionary
            Document

        Returns
        -------
        True: Success False: Failure
        """

        self._db_name = db_name
        self._collection_name = collection_name
        rtn = False

        # Do NOT export if the DB, collection are not specified
        if len(db_name) == 0 or len(collection_name) == 0:
            logger.error("DB param is empty.")
            return rtn

        # Insert the one document in the specified DB
        try:
            self._db = self._client[db_name]
            co = self._db[collection_name]
            ins_one_result = co.insert_one(doc)
            if ins_one_result.acknowledged:
                rtn = True
        except Exception:
            logger.error("Unable to insert data to NoSQL")

        return rtn

    # def _commit(self):

    #     try:
    #         self._session.commit_transaction()
    #     except Exception as e:
    #         self._session.abort_transaction()

    # def _abort(self):

    #     self._session.abort_transaction()

    # def _end_session(self):

    #     self._session.end_transaction()

    def close(self):
        """
        MongoDB client close
        """
        if self._client:
            self._client.close()
            self._client = None


if __name__ == '__main__':
    pass
