#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-
"""
 Connect to and Export from DB

 @author East Japan Accounting Center Co., Ltd

 (c)2019 The University of Aizu
 This software is released under the New BSD License.
"""
import logging
import psycopg2
import psycopg2.extras


logger = logging.getLogger(__name__)

# File R/W
FILE_MODE_WRITE = "w"
TABLE_SEARCH_QUERY = "SELECT relname FROM pg_class WHERE relkind = 'r' AND relname =%s"


class PostgreSQL:
    """
    PostgreSQL operation(Connection, File export, ...)
    """

    def __init__(self, host, port, dbname, tbl_name, db_user, db_password):
        self._db_name = dbname
        self._tbl_name = tbl_name
        self._db_user = db_user
        self._db_password = db_password
        self._host = host
        self._port = port
        self._db_client = None
        self._db_cursor = None

    def get_connection(self):
        """
        Connect to specified DB

        Returns
        -------
        DB instance
        """
        self._db_client = None

        try:
            doc = "dbname={} host={} user={} password={}".format(self._db_name, self._host, self._db_user, self._db_password)
            self._db_client = psycopg2.connect(doc)
        except psycopg2.Error as e:
            logger.error(e)
            logger.error("Unable to connect to RDB")
        return self._db_client

    def get_acquisition_cursor(self, conn):
        """
        acquisition a cursor instance from a destination

        Parameters
        ----------
        conn: connection object
            DB connection instance

        Returns
        -------
        cursor instance
        """
        self._db_cursor = None
        if conn is not None:
            try:
                self._db_cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            except psycopg2.Error as e:
                logger.error(e)

        return self._db_cursor

    def select(self, cur, sql, parameter):
        """
        Data select

        Parameters
        ----------
        cur: cursor
            cursor instance
        sql: string
            sql comand
        parameter: string
            select parameters

        Returns
        -------
        rtn: boolean
            select result
            True: Success
            False: Failure
        dict_data_list: list
            find fetch data
        """
        rtn = False
        dict_data_list = []
        # Validating table
        if not self._table_check(cur):
            logger.error("Unable to search from RDB")
            return rtn, dict_data_list
        sql = sql.format(self._tbl_name, parameter)
        try:
            cur.execute(sql,)
        except psycopg2.Error:
            logger.error("Unable to search from RDB")
        else:
            if cur.rowcount > 0:
                dict_data_list = [dict(fetch_data) for fetch_data in cur.fetchall()]
            rtn = True
        return rtn, dict_data_list

    def insert(self, cur, sql, parameter):
        """
        Data insert

        Parameters
        ----------
        cur: cursor
            cursor instance
        sql: string
            sql comand
        parameter: string
            insert parameters

        Returns
        -------
        rtn: boolean
            insert result
            True: Success
            False: Failure
        """
        rtn = False
        # Validating table
        if not self._table_check(cur):
            return rtn
        sql = sql.format(self._tbl_name)
        try:
            if type(parameter) is list:
                psycopg2.extras.execute_values(cur, sql, parameter)
            else:
                cur.execute(sql, parameter)
        except psycopg2.Error as e:
            logger.error(e)
            logger.error("Unable to insert data to RDB")
        else:
            if cur.rowcount > 0:
                rtn = True
        return rtn

    def _table_check(self, cur):
        """
        Validating table names

        Parameters
        ----------
        cur: cursor object
            DB cursor instance

        Returns
        -------
        rtn: boolean
            Export result
            True: Success
            False: Failure
        """
        rtn = False
        if cur is not None:
            try:
                cur.execute(TABLE_SEARCH_QUERY, (self._tbl_name,))
                if cur.fetchone():
                    rtn = True
            except psycopg2.Error as e:
                logger.error(e)

        return rtn

    def _commit(self):

        try:
            self._db_client.commit()
        except Exception:
            self._db_client.rollback()

    def _rollback(self):

        self._db_client.rollback()

    def close(self):
        """
        PostgreSQL client close
        """
        if self._db_cursor:
            self._db_cursor.close()
            self._db_cursor = None

        if self._db_client:
            self._db_client.close()
            self._db_client = None


if __name__ == '__main__':
    pass
